#ifndef _PCONST_H
#define _PCONST_H	1

/* internal constants */

#define ERR_PRO_INVALID_ARGUMENT 1
#define ERR_PRO_FILE_NOT_FOUND 2
#define ERR_PRO_CANT_OPEN_FILE 3
#define ERR_PRO_TEMPLATE_SYNTAX_ERROR 4
#define ERR_PRO_NOT_ENOUGH_MEMORY 5

#endif /* pconst.h */

/*
 * Local Variables:
 * mode: c
 * End:
 */
